#!/usr/bin/env python

#-------------------------------------------------------------------------------
# Name:        Bornederecharge.py
# Purpose:     regulation de la puissance d une wallbox 
#
# NP 2023
#
#-------------------------------------------------------------------------------




import serial
import time
import string
import requests
import spidev
from RPi import GPIO


#lecture port serie
SERIAL='/dev/ttyAMA0'
ser = serial.Serial(SERIAL, baudrate=1200, bytesize=serial.SEVENBITS, parity=serial.PARITY_EVEN, stopbits=serial.STOPBITS_ONE, timeout=0, rtscts=1)

#configuration GPIO
#GPIO.setwarnings(False)
#GPIO.setmode(GPIO.BCM)
#GPIO.setup(broche_HPHC, GPIO.IN, pull_up_down=GPIO.PUD_UP)
#GPIO.setup(broche_coupure_HP, GPIO.IN, pull_up_down=GPIO.PUD_UP)

#Gestion courant
#Valeur du courant disponible pour la wallbox (0 au demarragage du systeme)
courant = 0 
#Marge de consommation
marge_courant = 0
courant_precedent = 0
hphc_precedent = 'HC'
heure = time.time()

#CNA
spi_max_speed = 4 * 1000000 # 4 MHz
V_Ref = 3300 # 3V3 in mV
Resolution = 2**12 # 10 bits for the MCP 4911 - 12 bits pour le 4921
CE = 0 # CE0 or CE1, select SPI device on bus
DEBUG = False

#Configuration et ouverture port SPI
spi = spidev.SpiDev()
spi.open(0,CE)
spi.max_speed_hz = spi_max_speed

#Fonctions et sous-programmes



def lectureserie(ser):
	found_iinst = 0
	found_isousc = 0
	ok = 0
		
	while (ok != 1):	#Tant que les 2 valeurs (isousc et iinst) n'ont pas ete trouvees on effectue la boucle  
				
		linecont = ser.readline()
		if not linecont:
			xqz = 0 #print ('Empty: check wiring if multiple occurences')
		#Releve de la puissance souscrite - Variable local I1
		linecont1=str(linecont,'UTF-8')
		
		if ((str.find(linecont1,'ISOUSC'))!=-1 and found_isousc==0):
			try:
				I1 = int(linecont[7:9])
				if I1>=1:
					found_isousc = 1 
					
					
			except ValueError:
				print ('error isousc')
				
		
		#Releve de la puissance instantanee - Variable local I2
		if ((str.find(linecont1,'IINST'))!=-1 and found_iinst==0):
			try:
				I2 = int(linecont[6:9])
				if I2>=1:
					found_iinst = 1
					
										
			except ValueError:
				print ('error iinst')
		
		#Lorsque les 3 valeurs ont ete releves on passe ok a 1 et on envoie le resultat
		if (found_iinst & found_isousc):
			ok = 1
			
			return I1, I2
	
		time.sleep(0.15)

def calculcourant(courant, courant_charge_max): #Calcul du courant disponible a regler en sortie de la wallbox
		
		courant = courant_charge_max - (iinst-(isousc-marge_courant)) # Calcul de la nouvelle valeur du courant
				
		if (courant > courant_charge_max): #Si le courant depasse le courant max 
			courant = courant_charge_max # on ecrete au courant max
		if (courant < 6): #Si le courant disponible est inferieur a 6A on coupe la sortie de la wallbox (courant a zero)
			courant = 0
		
		return courant
	
def setOutput(courant):
	val = coefimage(courant) #convertie le courant en un coeficient de valeur binaire
    # lowbyte has 6 data bits
    # B7, B6, B5, B4, B3, B2, B1, B0
    # D5, D4, D3, D2, D1, D0,  X,  X
	lowByte = val << 0 & 0b11111111 #Mise en forme des bits de poids faibles (si MCP4911 mettre 2 a la place de 0)
    # highbyte has control and 4 data bits
    # control bits are:
    # B7, B6,   B5, B4,     B3,  B2,  B1,  B0
    # W  ,BUF, !GA, !SHDN,  D9,  D8,  D7,  D6
    # B7=0:write to DAC, B6=0:unbuffered, B5=1:Gain=1X, B4=1:Output is active
	highByte = ((val >> 8) & 0xff) | 0b0 << 7 | 0b0 << 6 | 0b1 << 5 | 0b1 << 4 #Mise en forme des bits de poids forts
    # by using spi.xfer2(), the CS is released after each block, transferring the
    # value to the output pin.
	if DEBUG :
		print("Highbyte = {0:8b}".format(highByte))
		print("Lowbyte =  {0:8b}".format(lowByte))
	spi.xfer2([highByte, lowByte])

def coefimage(courant):
	
	if (courant == 0):
		coef_courant = 0
	else:
		coef_courant = int(((courant * 0.0504125)+0.5)/((V_Ref*10**-3)/Resolution)) #Calcul du coeficient binaire correspondant au courant (formule deduite pour un evse viridian 32A)
	if DEBUG : print ("Binary value is : {0:12b} (12 bit)".format(coef_courant))
	# with a DAC gain of 1x
	print ("La tension de sortie devrait etre : {0} mV".format(coef_courant * V_Ref / Resolution))
	return coef_courant
 




while True:
	
	isousc, iinst = lectureserie(ser)
	
	
	
	
	courant_charge_max = 32
	courant = calculcourant(courant, courant_charge_max)
	print ("Courant de charge :", courant)
			

			
	if (courant != courant_precedent or (time.time()-heure > 3600)): 
		courant_precedent = courant
		heure=time.time()
		setOutput(courant)
			

